#!/usr/bin/python

import os
import time
from ConfigParser import ConfigParser
from subprocess import Popen

from fjd.core_process import CoreProcess
from fjd.utils import ensure_wdir


class Worker(CoreProcess):
    '''
    A worker process
    '''
    
    def __init__(self, interval=5, project=None):
        if not project:
            project = 'default'
        self.wdir = ensure_wdir(project)
        self.start_up()

        # announce my presence
        self.id = self.mk_id()
        print('[FJD] Worker with ID {id} started.'.format(id=self.id))
        os.system('touch {wdir}/workerqueue/{id}.worker'\
                   .format(wdir=self.wdir, id=self.id))

        while True:
            job = self.next_job_on_pod()
            if job:
                print('[FJD] Worker {}: I found a job.'.format(self.id))
                # A job is a config file
                conf = ConfigParser() 
                conf.read('{}/jobpod/{}'.format(self.wdir, job))
                exe = conf.get('control', 'executable') 
                log = conf.get('control', 'logfile') 
                # remove job from pod, execute task and re-announce myself
                cmd = 'touch {log}; {exe} {wdir}/jobpod/{job}; rm {wdir}/jobpod/{job}; '\
                      'touch {wdir}/workerqueue/{id}.worker'.format(exe=exe,
                                    job=job, wdir=self.wdir, log=log, id=self.id)
                Popen(cmd, shell=True).wait()
                print('[FJD] Worker {}: Finished my job.'.format(self.id))
            time.sleep(interval)

    def mk_id(self):
        host = os.uname()[1]
        ms = time.time()
        return '{h}_{t}'.format(h=host, t=ms)

    def next_job_on_pod(self):
        my_jobs = [j for j in os.listdir('{}/jobpod'.format(self.wdir))\
                  if j.startswith(self.id)]
        if len(my_jobs) > 0:
            return my_jobs[0]
        else:
            return None

